/*
 * Copyright © 2016 - 2017 Copyright (c) Sprint, Inc. and others.  All rights reserved.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */
package org.onosproject.fpcagent.util.eventStream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.AsyncContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletOutputStream;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;


/**
 * A servlet context listener that sends response (ConfigureOutput) events to clients.
 */
public class ConfigureService implements ServletContextListener {

    private static final Logger log = LoggerFactory.getLogger(ConfigureService.class);
    //key = client-id, value = configure-output
    public static BlockingQueue<Map.Entry<String, String>> configureQueue = new LinkedBlockingQueue<>();
    public static BlockingQueue<Map.Entry<String, String>> registerClientQueue = new LinkedBlockingQueue<>();

    public void contextInitialized(ServletContextEvent sce) {
        Map<String, AsyncContext> responseStreams = new ConcurrentHashMap<>();
        sce.getServletContext().setAttribute("responseStreams", responseStreams);
        Executors.newSingleThreadExecutor().submit(() -> {
            String clientUri = null;
            AsyncContext asyncContext = null;
            while (true) {
                try {
                    Map.Entry<String, String> entry = configureQueue.take();
                    clientUri = entry.getKey();
                    String notify = entry.getValue();

                    asyncContext = responseStreams.get(clientUri);
                    if (responseStreams.get(clientUri) != null) {
                        ServletOutputStream out = asyncContext.getResponse().getOutputStream();
                        out.write(notify.getBytes());
                        out.flush();
                        asyncContext.getResponse().flushBuffer();
                    }
                } catch (Exception e) {
                    log.error(ExceptionUtils.getFullStackTrace(e));
                    asyncContext.complete();
                    responseStreams.remove(clientUri);
                    break;
                }
            }
        });

        Executors.newSingleThreadExecutor().submit(() -> {
            String clientUri = null;
            AsyncContext asyncContext = null;
            while (true) {
                try {
                    Map.Entry<String, String> entry = registerClientQueue.take();
                    clientUri = entry.getKey();
                    asyncContext = responseStreams.get(clientUri);
                    if (responseStreams.get(entry.getKey()) != null) {
                        ServletOutputStream out = asyncContext.getResponse().getOutputStream();
                        out.write(entry.getValue().getBytes());
                        out.flush();
                        asyncContext.getResponse().flushBuffer();
                    }
                } catch (Exception e) {
                    log.error(ExceptionUtils.getFullStackTrace(e));
                    asyncContext.complete();
                    responseStreams.remove(clientUri);
                    break;
                }
            }
        });
    }

    public void contextDestroyed(ServletContextEvent sce) {
    }
}
